@@ -1,4 +1,4 @@
-# Copyright 2009, 2013 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -16,8 +16,5 @@
 
 """This package consists of the component framework used by Reahl Components.
 
-from __future__ import unicode_literals
-from __future__ import print_function
-import six
 It includes all component-related functionality needed in a production environment.
 """
