@@ -1,4 +1,4 @@
-# Copyright 2008-2013 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -14,19 +14,27 @@
 #    You should have received a copy of the GNU Affero General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""The Reahl configuration utilities."""
+"""Classes in this module manage system configuration.
+
+Each Reahl component can have its own config file. The config files of
+all components used by an application are stored in a common
+directory. If the config file for a component is missing, defaults are
+assumed.
+
+Config files are Python code. Inside a config file, an instance of the
+component's Configuration is bound to a variable name.
+
+"""
 
-from __future__ import unicode_literals
-from __future__ import print_function
 import sys
 import os.path
 import logging
-import tempfile
-from logging import config
+import locale
+import inspect
+import logging.config
 from contextlib import contextmanager
-import imp
 
-from pkg_resources import require, iter_entry_points, DistributionNotFound
+from pkg_resources import require, iter_entry_points, DistributionNotFound, Requirement
 
 from reahl.component.eggs import ReahlEgg
 from reahl.component.context import ExecutionContext
@@ -35,29 +43,41 @@ from reahl.component.context import ExecutionContext
 class ConfigurationException(Exception):
     pass
 
-class ExplicitSettingRequired(object):
+
+class ExplicitSettingRequired:
     pass
 
-class ConfigSetting(object):
+
+class ConfigSetting:
     """Used to define one configuration setting on a :class:`Configuration`.
     
     
-       :param default: The default value to be used if not specified in a config file.
-       :param description: A user readable description explaining what this ConfigSetting is for.
-       :param dangerous: Set this to True, if a warning should be emitted when used with the supplied default value.
-       :param automatic: Set this to True for a ConfigSetting which is meant to be used for dependency injection.
+       :keyword default: The default value to be used if not specified in a config file.
+       :keyword description: A user readable description explaining what this ConfigSetting is for.
+       :keyword dangerous: Set this to True, if a warning should be emitted when used with the supplied default value.
+       :keyword automatic: Set this to True for a ConfigSetting which is meant to be used for dependency injection.
     """
     _name = None
+
     def __init__(self, default=ExplicitSettingRequired, description='No description supplied', dangerous=False, automatic=False):
         self.description = description
         self.default = default
         self.dangerous = dangerous
         self.automatic = automatic
 
+    def __str__(self):
+        return 'for humans'
+
     @property
     def defaulted(self):
         return self.default is not ExplicitSettingRequired
 
+    def default_value_for_configuration(self, config):
+        if isinstance(self.default, DeferredDefault):
+            return self.default(config)
+        else:
+            return self.default
+        
     def is_localised(self, obj):
         name = self.name(type(obj))
         for key in dir(obj):
@@ -79,7 +99,7 @@ class ConfigSetting(object):
             return obj.__dict__[setting_name]
 
         if self.defaulted:
-            return self.default
+            return self.default_value_for_configuration(obj)
 
         raise ConfigurationException('%s was not set' % setting_name)
 
@@ -95,7 +115,7 @@ class ConfigSetting(object):
                 if value is self:
                     self._name = name
                     return name
-        raise AssertionError('Could not deduce name for descriptor %s (%s) %s' % (self, self.description, self.default))
+        raise AttributeError('Could not deduce name for descriptor %s (%s) %s' % (self, self.description, self.default))
 
     def is_set(self, obj):
         name = self.name(type(obj))
@@ -107,15 +127,32 @@ class ConfigSetting(object):
         return self.dangerous and not self.is_set(obj)
 
 
+class DeferredDefault:
+    """Sometimes the default value for a :class:`ConfigSetting` cannot be set when the :class:`Configuration`
+       is declared. An instance of DeferredDefault can be passed as default in such a scenario.
+
+       DeferredDefault wraps a callable, which will be called only once the config value is read. The callable 
+       is passed a single argument: the Configuration on which this ConfigSetting is defined.
+    """
+    def __init__(self, getter):
+        self.getter = getter
+
+    def __call__(self, config):
+        return self.getter(config)
+
+    def __str__(self):
+        return 'DeferredDefault(%s)' % inspect.getsource(self.getter).strip()
+
+    
 class EntryPointClassList(ConfigSetting):
     """A :class:`ConfigSetting` which is not set by a user at all -- rather, its value (a list of classes
        or other importable Python objects) is read from the entry point named `name`.
 
        :param name: The name of the entry point to read.
-       :param description: (See :class:`ConfigSetting`)
+       :keyword description: (See :class:`ConfigSetting`)
     """
     def __init__(self, name, description='Description not supplied'):
-        super(EntryPointClassList, self).__init__(default=[], description=description)
+        super().__init__(default=[], description=description)
         self.name = name
         
     def __get__(self, instance, owner):
@@ -132,12 +169,12 @@ class EntryPointClassList(ConfigSetting):
         return classes
 
 
-class MissingValue(object):
+class MissingValue:
     def __repr__(self):
-        return _('MISSING!!')
+        return 'MISSING!!'
 
     
-class Configuration(object):
+class Configuration:
     """A collection of ConfigSettings for a component. To supply configuration for your component,
        subclass from this class and assign each wanted ConfigSetting as a class attribute. Assign
        the required `filename` and `config_key` class attributes in your subclass as well. The resultant
@@ -182,10 +219,11 @@ class Configuration(object):
         for name, config_item in self.config_items():
             composite_name = '%s.%s' % (composite_key, name)
             if config_item.is_dangerous(self):
+                message = '%s has been defaulted to a value not suitable for production use: "%s". You can set it in %s' % (composite_name, config_item.default, filename)
                 if strict_validation:
-                    raise ConfigurationException('%s in %s is using a dangerous default setting: %s' % (composite_name, filename, config_item.default))
+                    raise ConfigurationException(message)
                 else:
-                    logging.getLogger(__name__).warning('%s in %s is using a dangerous default setting: %s' % (composite_name, filename, config_item.default))
+                    logging.getLogger(__name__).warning(message)
 
     def list_contents(self, filename, composite_key):
         contents = []
@@ -206,7 +244,7 @@ class Configuration(object):
         pass
         
 
-class NullORMControl(object):
+class NullORMControl:
     def do_nothing(self, *args, **kwargs):
         pass
 
@@ -222,9 +260,10 @@ class ReahlSystemConfig(Configuration):
     filename = 'reahl.config.py'
     config_key = 'reahlsystem'
     root_egg = ConfigSetting(description='The root egg of the project', default=os.path.basename(os.getcwd()), dangerous=True)
-    connection_uri = ConfigSetting(description='The database connection URI', default='sqlite:///%s' % os.path.join(tempfile.gettempdir(), 'reahl.db'), dangerous=True)
+    connection_uri = ConfigSetting(description='The database connection URI',
+                                   default=DeferredDefault(lambda c: 'sqlite:///%s.db' % (os.path.join(os.getcwd(), Requirement.parse(c.root_egg).project_name))), dangerous=True)
     orm_control = ConfigSetting(default=NullORMControl(), description='The ORM control object to be used', automatic=True)
-    debug = ConfigSetting(default=False, description='Enables more verbose logging')
+    debug = ConfigSetting(default=True, description='Enables more verbose logging', dangerous=True)
     databasecontrols = EntryPointClassList('reahl.component.databasecontrols', description='All available DatabaseControl classes')
     translation_packages = EntryPointClassList('reahl.translations', description='All available packages containing translation messages')
     serialise_parallel_requests = ConfigSetting(default=False, description='Whether concurrent requests to the web application should be forcibly serialised')
@@ -232,6 +271,7 @@ class ReahlSystemConfig(Configuration):
 
 class ConfigAsDict(dict):
     def __init__(self, config):
+        super().__init__()
         self.config = config
         self['config'] = config
 
@@ -251,15 +291,23 @@ class ConfigAsDict(dict):
 
 
 class StoredConfiguration(Configuration):
-    def __init__(self, config_directory_name, in_production=False):
+    def __init__(self, config_directory_name, strict_checking=False):
         self.config_directory = config_directory_name
-        self.in_production = in_production
+        self.strict_checking = strict_checking
 
+    def check_for_python_issue_18378(self):
+        try:
+            locale.getdefaultlocale()
+        except ValueError:
+            if sys.platform == 'darwin':
+                logging.getLogger(__name__).warning('WARNING: It looks like your system is suffering from the MacOS locale bug'
+                                                    ' (https://bugs.python.org/issue18378).'
+                                                    ' Setting your locale to en_US.utf8 to bypass.')
+                os.environ['LC_ALL'] = 'en_US.UTF-8'
+                os.environ['LANG'] = 'en_US.UTF-8'
+                
     def configure(self, validate=True):
-        #http://mail.python.org/pipermail/tutor/2005-August/040993.html
-        imp.reload(sys); #read setdefaultencoding python docs - it "enables" the method again
-        sys.setdefaultencoding('utf-8')
-        
+        self.check_for_python_issue_18378()
         self.configure_logging()
         logging.getLogger(__name__).info('Using config in %s' % self.config_directory)
         sys.path.insert(0,self.config_directory)
@@ -270,13 +318,7 @@ class StoredConfiguration(Configuration):
         self.read(ReahlSystemConfig)
         self.validate_required(ReahlSystemConfig)
 
-        try:
-            require(self.reahlsystem.root_egg)
-        except DistributionNotFound as ex:
-            requirement = ex.args[0]
-            if (requirement.project_name == self.reahlsystem.root_egg.replace('_','-')) and not self.in_production:
-                ex.args = ('%s (It looks like you are in a development environment. Did you run "reahl setup -- develop -N"?)' % ex.args[0],)
-            raise
+        require(self.reahlsystem.root_egg)
 
         self.configure_components()
         if validate:
@@ -286,7 +328,7 @@ class StoredConfiguration(Configuration):
     def configure_logging(self):
         logging_config_file = os.path.join(self.config_directory, 'logging.conf')
         if os.path.isfile(logging_config_file):
-            config.fileConfig(logging_config_file)
+            logging.config.fileConfig(logging_config_file)
         else:
             logging.basicConfig()
         logging.captureWarnings(True)
@@ -304,19 +346,20 @@ class StoredConfiguration(Configuration):
             egg.validate_config(self)
 
     def list_all(self):
-        all = []
-        all.extend(self.list_required(ReahlSystemConfig))
+        all_items = []
+        all_items.extend(self.list_required(ReahlSystemConfig))
         eggs = ReahlEgg.get_all_relevant_interfaces(self.reahlsystem.root_egg)
         for egg in reversed(eggs):
-            all.extend(egg.list_config(self))
-        return all
+            all_items.extend(egg.list_config(self))
+        return all_items
 
     def read(self, configuration_class):
         new_config = self.create_config(configuration_class)
         file_path = os.path.join(self.config_directory, new_config.filename)
         if os.path.isfile(file_path):
             locals_dict = ConfigAsDict(self)
-            exec(compile(open(file_path).read(), file_path, 'exec'), globals(), locals_dict)
+            with open(file_path) as f:
+                exec(compile(f.read(), file_path, 'exec'), globals(), locals_dict)
             locals_dict.update_required(new_config.config_key)
         else:
             message = 'file "%s" not found, using defaults' % file_path
@@ -352,7 +395,7 @@ class StoredConfiguration(Configuration):
         if not isinstance(src_config, configuration_class):
             raise ConfigurationException('%s is not a %s in %s' % (composite_key, configuration_class, full_filename))
 
-        src_config.validate_contents(full_filename, composite_key, self.in_production)
+        src_config.validate_contents(full_filename, composite_key, self.strict_checking)
 
     def list_required(self, configuration_class):
         composite_key = configuration_class.config_key
@@ -369,10 +412,10 @@ class StoredConfiguration(Configuration):
 
 class CodedConfiguration(StoredConfiguration, dict):
     def __init__(self):
-        super(CodedConfiguration, self).__init__('<in memory>')
+        super().__init__('<in memory>')
 
     def read(self, configuration_class):
-        fimename = configuration_class.filename
+        filename = configuration_class.filename
         new_config = self.create_config(configuration_class)
         if filename in self:
             locals_dict = ConfigAsDict(self)
